/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie Server.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * Poddie Server is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * A copy of the GNU Affero General Public License can be found in the
 * file LICENSE. 
 */

/*
 *                         application.cpp  -  description
 *                         --------------------------------
 *   begin                : Tue Aug 13 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : This is the starting point for the
 *                          application. The game core and other
 *                          essentials are setup here.
 *
 */

#include "application.h"

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                                  MAIN
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

Application app; //Create an instance of this application.

int Application::main(int argc, char** argv) {
	
	//Redirect errors to the console.
	CL_ConsoleWindow console("Debug console window", 80, 25);
	console.redirect_stdio();
	
	try {
		std::cout << "Starting Poddie Server... \t";
		
		//Setup ClanLib core
		CL_SetupCore setup_core;
		
		std::cout << "Success" << std::endl;
		std::cout << "Setting up network... \t";
		
		//Setup the networking
		CL_SetupNetwork setup_network;
		
		std::cout << "Success" << std::endl;
		std::cout << "Creating game world... \t";
		
		//Create the world
		World world("7223");
		
		std::cout << "Success" << std::endl;
		std::cout << "Starting game world now." << std::endl;
		
		try {
			world.run(); // Begin the game loop.
		}
		catch (Exception &e) {
			cerr << "Error: " << e.message() << " (" << e.what() << ")" << endl;
		}
		
	}
		
	catch (CL_Error err) {
		std::cout << "Exeception caught: " << err.message.c_str() << std::endl;
		console.wait_for_key();
	}
	return 0;
}
