/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie Server.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * Poddie Server is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * A copy of the GNU Affero General Public License can be found in the
 * file LICENSE. 
 */

/*
 *                         exceptionhandler.h  -  description
 *                         --------------------------------
 *   begin                : Sun Sep 20 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : Exception classes are kept in this file.
 *                          These classes takes care of exception
 *                          handling.
 *
 */

/**
 * We use throw() to prevent the functions from throwing exceptions.
 */

class Exception : public std::exception {
private:
	const string _message;
	mutable string _what;
	const exception &operator = (const Exception&);

protected:
	Exception(const string &message) throw() : _message(message) { }

	Exception(const Exception &other) : std::exception(other), _message(other._message) { }

public:
	virtual ~Exception() throw() { }
	
	//Returns the error message
	const string &message() const throw() { return _message; }
	
	//Returns the type of the error.
	const char* what() const throw() {
		int s(0);

		char* const name(abi::__cxa_demangle((std::string("_Z") + typeid(*this).name()).c_str(), 0, 0, &s));

		if (s == 0) {
			_what = name;
			free(name);
		}

		if (_what.empty()) _what = convert::stringify(std::exception::what());

		return _what.c_str();
	}
};
