/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie Server.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * Poddie Server is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * A copy of the GNU Affero General Public License can be found in the
 * file LICENSE. 
 */

/*
 *                         component.cpp  -  description
 *                         --------------------------------
 *   begin                : Tue Aug 16 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : Defines all components' functions, etc.
 *
 */

#include "component.h"

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                            CON- & DESTRUCTORS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

Component::Component(int slot, string type, string part, string number, Player *owner, World *world) :
slot(slot), type(type), part(part), number(number), owner(owner), world(world) {
	active = false;
	increase = false;
	decrease = false;
	setPrevActivity = false;
	activity = 0;
	prevActivity = 0;
	//cout << "Added component: \"Poddie/" << type << "/" << part << "/" << number << "\" to Player " << owner->cid << endl;
}

Component::~Component() {
	
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                            LOOPING FUNCTIONS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

void Component::update() {
	
	////////////////////////////////////////////////////////////////////
	//!                    COMPONENT FUNCTIONS
	////////////////////////////////////////////////////////////////////
	
	////////////////////////////////////////////////////////////////////
	//                           RUSTY
	
	if (type == "Rusty") {
		if (part == "Engine") {
			if (number == "BackCenter1") {
				if (active) owner->add_vector(owner->get_angle(), activity/40);
				if (increase) activity += 0.02;
				if (decrease) activity -= 0.02;
			}
			if (number == "BackRight1") {
				if (active) { owner->add_turn(-activity/500); owner->add_vector(owner->get_angle(), activity/50); }
				if (increase) activity += 0.02;
				if (decrease) activity -= 0.02;
			}
			if (number == "BackLeft1") {
				if (active) { owner->add_turn(activity/500); owner->add_vector(owner->get_angle(), activity/50); }
				if (increase) activity += 0.02;
				if (decrease) activity -= 0.02;
			}
			if (number == "MiddleRight1") {
				if (active) { owner->add_turn(-activity/200); owner->add_vector(owner->get_angle(), activity/60); }
				if (increase) activity += 0.02;
				if (decrease) activity -= 0.02;
			}
			if (number == "MiddleLeft1") {
				if (active) { owner->add_turn(activity/200); owner->add_vector(owner->get_angle(), activity/60); }
				if (increase) activity += 0.02;
				if (decrease) activity -= 0.02;
			}
		}
		
		if (part == "Weapon/Laser") {
			if (number == "MiddleRight1") {
				if (active && CL_System::get_time() / 1000.0f >  cooldown) {
					active = false;
					
					float x = owner->x+cos((owner->angle+56.24)*DEG_TO_RAD)*359;
					float y = owner->y+sin((owner->angle+56.24)*DEG_TO_RAD)*359;
					
					Missile *newest;
					string sprite = "Missile/Laser/Green/Small1";
					
					world->missile.push_back(new Missile(x, y, owner->angle, 100, owner, owner->world, sprite));
					newest = world->missile.at(world->missile.size()-1);
					world->network_missile_create_send(true, owner->cid, x, y, owner->angle,
														newest->hspeed, newest->vspeed, sprite);
					
					// Set the cooldown:
					cooldown = CL_System::get_time() / 1000.0f + /* Cooldown in seconds: */ 1.0f;
				}
			}
			if (number == "MiddleLeft1") {
				if (active && CL_System::get_time() / 1000.0f >  cooldown) {
					active = false;
					
					float x = owner->x+cos((owner->angle-56.24)*DEG_TO_RAD)*359;
					float y = owner->y+sin((owner->angle-56.24)*DEG_TO_RAD)*359;
					
					Missile *newest;
					string sprite = "Missile/Laser/Green/Small1";
					
					world->missile.push_back(new Missile(x, y, owner->angle, 100, owner, owner->world, sprite));
					newest = world->missile.at(world->missile.size()-1);
					world->network_missile_create_send(true, owner->cid, x, y, owner->angle,
														newest->hspeed, newest->vspeed, sprite);
					
					// Set the cooldown:
					cooldown = CL_System::get_time() / 1000.0f + /* Cooldown in seconds: */ 1.0f;
				}
			}
		}
	}
	
	////////////////////////////////////////////////////////////////////
	//!                         ACTIVITY
	////////////////////////////////////////////////////////////////////
	
	if (activity != prevActivity) {
		owner->world->network_component_activity_send(owner->cid, activity, slot);
		prevActivity = activity;
	}
	
	if (active && setPrevActivity) prevActivity = activity;
	if (active && activity < 0.1) activity += 0.01;
	
	if (active && !setPrevActivity && activity < prevActivity) activity += 0.02;
	else if (active && activity >= prevActivity) setPrevActivity = true;
	if (!active) { activity -= 0.02; setPrevActivity = false; }
	
	if (activity > 1) activity = 1;
	if (activity < 0) activity = 0;
}
