/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie Server.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * Poddie Server is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * A copy of the GNU Affero General Public License can be found in the
 * file LICENSE. 
 */

/*
 *                         missile.cpp  -  description
 *                         --------------------------------
 *   begin                : Tue Aug 16 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : Any flying missile.
 *
 */

#include "missile.h"

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                            CON- & DESTRUCTORS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

Missile::Missile(float x, float y, float angle, float speed, Player *creator, World *world, string sprite) : GameObject(x, y, angle, world), sprite(sprite), creator(creator) {
	hspeed = 0;
	vspeed = 0;
	alive = true;
	add_vector(angle, speed);
	radius = speed/2;
	do_update = false;
}

Missile::Missile() {
}

Missile::~Missile() {
	
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                           LOOPING FUNCTIONS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

void Missile::update() {
	x += hspeed;
	y += vspeed;
	
	if (x < 0 || y < 0 || x > 2048*3*15 || y > 2048*3*15) alive = false;
}

/***********************************************************************
*    *    *    *    *                              *    *    *    *    *
                               FUNCTIONS
*    *    *    *    *                              *    *    *    *    *
***********************************************************************/

////////////////////////////////////////////////////////////////////////
//                     MOVEMENT, POSITION & DIRECTION

void Missile::add_vector(float angle, float acceleration) {
	vspeed += sin(angle*DEG_TO_RAD)*acceleration;
	hspeed += cos(angle*DEG_TO_RAD)*acceleration;
}

////////////////////////////////////////////////////////////////////////
//                              COLLSION

bool Missile::collision_circle(float center_x, float center_y, float radius) {
	if (sqrt(pow(x-center_x, 2)+pow(y-center_y, 2)) < radius) return true;
	else return false;
}
