/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie Server.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * Poddie Server is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * A copy of the GNU Affero General Public License can be found in the
 * file LICENSE. 
 */

/*
 *                         player.h  -  description
 *                         --------------------------------
 *   begin                : Tue Aug 13 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : Player object to control player variables,
 *                          components, etc.
 *
 */

/// //////////////////////////////////////////////////////////////// ///
///                          PREPROCESSOR                            ///
/// //////////////////////////////////////////////////////////////// ///

#ifndef header_player
#define header_player

#if _MSC_VER > 1000
#pragma once
#endif

#include "../precomp.h"
#include "gameobject.h"
#include "component.h"
#include "../world.h"

/// //////////////////////////////////////////////////////////////// ///
///                    YET TO BE DEFINED CLASSES                     ///
/// //////////////////////////////////////////////////////////////// ///

class Component;

/// //////////////////////////////////////////////////////////////// ///
///                           DEFINITION                             ///
/// //////////////////////////////////////////////////////////////// ///

class Player : public GameObject {
public:
	// Constructors
	Player(float x, float y, float angle, World *world, int cid);
	Player();
	// Destructor
	~Player();

	// Controller
	std::string controller;
	int cid;
	float timer_last_update;
	
	void set_controller(std::string new_controller);
	std::string get_controller();

	// Status
	bool alive;
	float health;
	float timer_respawn;
	bool do_respawn;

	// Update function
	void update();
	
	// Components
	std::list <Component*> components;
	std::list<Component*>::iterator component[16];
	bool slot[16];
	
	void add_component(list<Component*>::iterator, Component*);
	void remove_component(list<Component*>::iterator, int);
	
	// Movement, position & direction
	bool doesMove, doesTurn, turnRight;
	void respawn();
	
	float speed, turnSpeed, acceleration;
	float radius;
	
	// Simple Physics
	void add_vector(float, float);
	void add_turn(float);
	void apply_friction(float);
	void turn_right();
	void turn_left();
	void set_position(float, float);
};

#endif
