/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie Server.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * Poddie Server is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * A copy of the GNU Affero General Public License can be found in the
 * file LICENSE. 
 */

/*
 *                         precomp.h  -  description
 *                         --------------------------------
 *   begin                : Tue Aug 13 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : Contains includes, macro definitions and
 *                          conditionally includes that affect some or
 *                          all classes.
 *
 */

/// //////////////////////////////////////////////////////////////// ///
///                          PREPROCESSOR                            ///
/// //////////////////////////////////////////////////////////////// ///

#ifndef header_precomp
#define header_precomp

#ifdef WIN32
#pragma warning(disable : 4786)
#endif

////////////////////////////////////////////////////////////////////////
//                            CONSTANTS

/* Define constants to convert radians to degrees and vice versa.
 * Usage: <radian value> * RAD_TO_DEG = <radian value in degrees>
 */
#ifndef M_PI
#define M_PI 3.14159265358979323846 
#endif
#define DEG_CIRCLE 360
#define DEG_TO_RAD (M_PI / (DEG_CIRCLE / 2))
#define RAD_TO_DEG ((DEG_CIRCLE / 2) / M_PI)

// Define integer constants for objects.
#define PLAYER 1
#define MISSILE 2

////////////////////////////////////////////////////////////////////////
//                        EXTERNAL LIBRARIES

#include <ClanLib/core.h>
#include <ClanLib/network.h>
#include <ClanLib/application.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <stdlib.h>
#include <cmath>
#include <cstdio>
#include <cstdlib>
#include <vector> 
#include <cxxabi.h> //Exception names

////////////////////////////////////////////////////////////////////////
//                             NAMESPACES

namespace convert {
	template <typename T>
	
	// Converts any type that supports the operator << into a string.
	std::string stringify(const T &v) {
		std::ostringstream ss;

		ss << v;

		return ss.str();
	}

	template <>
	inline std::string stringify(const std::string &s) {
		return s;
	}
}

using namespace std;
using namespace convert;

////////////////////////////////////////////////////////////////////////
//                          GLOBAL HEADERS

#include "exceptionhandler.h"

#endif
