/* Copyright (C) 2009 Sakse Dalum
 * 
 * This file is part of Poddie Server.
 * 
 * Poddie Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * Poddie Server is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * A copy of the GNU Affero General Public License can be found in the
 * file LICENSE. 
 */

/*
 *                         world.h  -  description
 *                         --------------------------------
 *   begin                : Tue Aug 13 2009
 *   copyright            : (C) 2009 by Sakse Dalum
 *   maintainer(s)        : Sakse "Don S" Dalum <don.s@yodanism.org>
 *   description          : Controls all major functions of the game,
 *                          and takes care of updating objects etc.
 *
 */

/// //////////////////////////////////////////////////////////////// ///
///                          PREPROCESSOR                            ///
/// //////////////////////////////////////////////////////////////// ///

#ifndef header_world
#define header_world

#if _MSC_VER > 1000
#pragma once
#endif

#include "precomp.h"
#include "gameobjects/component.h"
#include "gameobjects/player.h"
#include "gameobjects/missile.h"

/// //////////////////////////////////////////////////////////////// ///
///                    YET TO BE DEFINED CLASSES                     ///
/// //////////////////////////////////////////////////////////////// ///

class Player;
class Missile;

/// //////////////////////////////////////////////////////////////// ///
///                           DEFINITION                             ///
/// //////////////////////////////////////////////////////////////// ///

class World {
public:
	// Constructor
	World(string);
	
	// Destructor
	~World();

	// Operations
	void run();
	bool quit;

	// Slots
	CL_SlotContainer slots;

	// In-game Objects
	vector< Player* > player;
	vector< Missile* > missile;
	
	// Map
	float border_right, border_left, border_top, border_bottom;
	
	// Networking
	int players_connected;
	
	float timer_update_player;
	float timer_update_missile;
	
	CL_NetSession *netsession; // The session we use when communicating.
	string port;
	
	// Packets
	CL_NetPacket *packet_object_status;
	CL_NetPacket *packet_missile_create;
	CL_NetPacket *packet_collision;
	CL_NetPacket *packet_component_add;
	CL_NetPacket *packet_component_remove;
	CL_NetPacket *packet_component_activate;
	CL_NetPacket *packet_component_activity;
	CL_NetPacket *packet_disconnect_player;
	CL_NetPacket *packet_delay;
	
	// Sending packets
	void network_object_status_send(int, int);
	void network_missile_create_send(bool, int, float, float, float, float, float, string);
	void network_collision_send(int, int);
	void network_component_add_send(int, string, string, string, int);
	void network_component_remove_send(int, int);
	void network_component_activate_send(int, string, bool, int);
	void network_component_activity_send(int, float, int);
	void network_disconnect_player_send(int);
	
	// Receiving packets
	void network_component_add_receive(CL_NetPacket &packet, CL_NetComputer &computer);
	void network_component_remove_receive(CL_NetPacket &packet, CL_NetComputer &computer);
	void network_component_activate_receive(CL_NetPacket &packet, CL_NetComputer &computer);
	void network_components_request_receive(CL_NetPacket &packet, CL_NetComputer &computer);
	void network_delay_return_receive(CL_NetPacket &packet, CL_NetComputer &computer);
	void network_disconnect_receive(CL_NetPacket &packet, CL_NetComputer &computer);
	
	// Connect/disconnect signals
	void network_connect(CL_NetComputer &computer);
	void network_disconnect(CL_NetComputer &computer);
	
	// Force disconnect player
	void disconnect_player(int);
};

#endif
